/*
 * iGeom by LInE
 * Geometric Object: Segment
 * Manage intersection of Segment with: Sements, Circumference
 * www.matematica.br/igeom
 * .app/components/line-segment-component/models/line-segment-model.js
 * @version 2020/11/02: Implemented Line instersection
 */

import { GeometricObject } from "../../../core/models/objects/geometric-object";
import { IntersectionModel } from "../../intersection-component/models/intersection-model";
import { ELEMENTS_CLASS } from "../../../core/enums/elements-class-enum";

export class LineSegmentModel extends GeometricObject {

  constructor (pointA, pointB, label, id) {
    super(id);
    this.pointA = pointA;
    this.pointB = pointB;
    this.setLabel(label);
    super.setClass(ELEMENTS_CLASS.LINE_SEGMENT);
    this.definitions.push(this.pointA);
    this.definitions.push(this.pointB);
    }

  getStraight () {
    const aX = this.pointA.posX;
    const aY = this.pointA.posY;
    const bX = this.pointB.posX;
    const bY = this.pointB.posY;
    const a = bY - aY;
    const b = aX - bX;
    const c = a * aX + b * aY;
    return [a, b, c];
    }

  // Return [(Bx-Ax, By-Ay)]
  getDirection () {
    const aX = this.pointA.posX;
    const aY = this.pointA.posY;
    const bX = this.pointB.posX;
    const bY = this.pointB.posY;
    const a = bX - aX;
    const b = bY - aY;
    return [a, b];
    }


  getMiddlePoint () {
    const x = (this.pointA.posX + this.pointB.posX) / 2;
    const y = (this.pointA.posY + this.pointB.posY) / 2;
    return { posX: x, posY: y };
    }

  getIntersection (geometricObject) {
    switch (geometricObject.elementClass) {
      case ELEMENTS_CLASS.LINE: // StraightLine
        return geometricObject.getIntersectionWithStraightLine(this); // Delegate to StraightLine
      case ELEMENTS_CLASS.LINE_SEGMENT:
        return this.getIntersectionByLine(geometricObject); //TODO nome deveria especificar SEGMENTO: this.getIntersectionWithSegment
      case ELEMENTS_CLASS.CIRCUMFERENCE:
        return this.getIntersectionWithCircumference(geometricObject);
      default:
        break;
      }
    }

  getDeterminantByLine (lineSegment) {
    const og1 = this.getStraight();
    const og2 = lineSegment.getStraight();
    const a1 = og1[0];
    const b1 = og1[1];
    const a2 = og2[0];
    const b2 = og2[1];
    const determinant = a1 * b2 - a2 * b1;
    return determinant;
    }

  getIntersectionByLine (lineSegment) { //TODO nome deveria especificar SEGMENTO: getIntersectionWithSegment
    const og1 = this.getStraight(); //r
    const og2 = lineSegment.getStraight(); //s
    const a1 = og1[0];
    const b1 = og1[1];
    const c1 = og1[2];
    const a2 = og2[0];
    const b2 = og2[1];
    const c2 = og2[2];
    const determinant = a1 * b2 - a2 * b1;
    if (determinant == 0) {
      return [new IntersectionModel(Number.MAX_SAFE_INTEGER, Number.MAX_SAFE_INTEGER, undefined, this, lineSegment, true, 0)];
      }
    else {
      const x = (b2 * c1 - b1 * c2) / determinant;
      const y = (a1 * c2 - a2 * c1) / determinant;
      return [new IntersectionModel(x, y, undefined, this, lineSegment, true, 0)];
      }
    }

  // Intersection with circunference
  // @calledby line-model.js!getIntersectionWithCircumference(circ): return super.getIntersectionWithCircumference(circ);
  getIntersectionWithCircumference (circumference) { //TODO Nome? Melhor 'getIntersectionWithCircumference'
    const pointA = this.pointA;
    const pointB = this.pointB;
    const center = circumference.center;
    const radiusP = circumference.getRadius(); // is 'circumference.radius' that is a Point
    const radius = circumference.getRadiusValue(); // is '||circumference.radius-circumference.center||
    var existA = -1, existB = -1;
    var aux = " (pointA.id=" + pointA.id + ", pointB.id=" + pointB.id + ", radiusP.id=" + radiusP.id + ")";
    if (pointA.id==radiusP.id) {
      console.log("*** line-segment-model.js!getIntersectionWithCircumference(.): PointA already in intersection!" + aux);
      existA = 1; //pointA.og1 = this; pointA.og2 = circumference;
      }
    else
    if (pointB.id==radiusP.id) {
      console.log("*** line-segment-model.js!getIntersectionWithCircumference(.): pointA already in intersection!" + aux);
      existB = 1; //pointB.og1 = this; pointB.og2 = circumference;
      }
    //D else console.log("xxx line-segment-model.js!getIntersectionWithCircumference(.): pointA or PointB NOT in intersection!" + aux);
    //D console.trace();
    const dx = pointB.posX - pointA.posX; // segment line pointA
    const dy = pointB.posY - pointA.posY; // 
    const cx = center.posX - pointA.posX; // circumference center point (center)
    const cy = center.posY - pointA.posY; //

    const a = dx * dx + dy * dy; // ||B-A||^2 = (B-A)'(B-A)
    const b = dx * cx + dy * cy; // (B-A)'(C-A)
    const c = cx * cx + cy * cy - radius * radius; // ||C-A||^2 - r^2
    const D = b / a; // (B-A)'(C-A) / (B-A)'(B-A)
    const q = c / a; // (||C-A||^2 - r^2) / (B-A)'(B-A)

    const delta = D * D - q;

    var PA, PB;
    if (existA==-1) PA = new IntersectionModel(Number.MAX_SAFE_INTEGER, Number.MAX_SAFE_INTEGER, undefined, this, circumference, false, 0);
    else            PA = pointA; // intersection already exist (with point A)
    if (existB==-1) PB = new IntersectionModel(Number.MAX_SAFE_INTEGER, Number.MAX_SAFE_INTEGER, undefined, this, circumference, false, 1);
    else            PA = pointB; // intersection already exist (with point B)

    if (delta < 0) { //TODO Verificar? No intersection points
      //D console.log("line-segment-model.js: getIntersectionWithCircumference(.): delta<0: " + Number.MAX_SAFE_INTEGER);
      return [PA, PB];
      }

    const deltaSqrt = Math.sqrt(delta);
    const root1 = -D + deltaSqrt;
    const root2 = -D - deltaSqrt;
    const x1 = pointA.posX - dx * root1
    const y1 = pointA.posY - dy * root1;

    if (delta == 0) { //TODO Verificar? Only one point (actually, both with the same coordinates)
      //D console.log("line-segment-model.js: getIntersectionWithCircumference(.): delta==0");
      if (existA==-1)
        PA.bind(x1, y1, undefined, this, circumference, true, 0);
      return [PA, PB];
      }

    const x2 = pointA.posX - dx * root2;
    const y2 = pointA.posY - dy * root2;

    if (existA==-1) PA.bind(x1, y1, undefined, this, circumference, true, 0);
    if (existB==-1) PB.bind(x2, y2, undefined, this, circumference, true, 1);

    if (!this.insideSegment(PB.posX, PB.posY)) {
      if (existB==-1) PB.bind(Number.MAX_SAFE_INTEGER, Number.MAX_SAFE_INTEGER, undefined, this, circumference, false, 1);
      }

    return [PA, PB];
    } // getIntersectionWithCircumference(circumference)


  // Considering de "level set" defined by direction d=B-A and intersection P,
  // if d'P<d'A or d'P>d'B, then intersection is empty
  insideSegment (intersecX, intersecY) {
    //D console.log("line-segment-model.js: insideSegment(.)");
    const valuesR = this.getDirection(); // get d=B-A
    const dirX = valuesR[0]; // Bx-Ax
    const dirY = valuesR[1]; // By-Ay
    const cInterA = dirX * intersecX + dirY * intersecY;

    // comparacao cv do ponto A > cv da intersec =>  vazio
    const cRA = dirX * this.pointA.posX + dirY * this.pointA.posY;

    if (cInterA < cRA) { // d'P < d'A
      return false;
      }

    // comparacao cv do ponto B < cv da intersec =>  vazio
    const cRB = dirX * this.pointB.posX + dirY * this.pointB.posY;

    if (cInterA > cRB) { // d'P > d'B
      this.posX = Number.MAX_SAFE_INTEGER;
      this.posY = Number.MAX_SAFE_INTEGER;
      return false;
      }

    return true;
    }


  // Help: search object in more stable structure
  static findOD_LS (vecIdObj, id) {
    var tam = vecIdObj.length, ii;
    for (ii=0; ii<tam; ii++)
      if (vecIdObj[ii].id === id) return ii;
    return -1; // Error! Not found!
    }

  // In reading GEO file, need to transform a Map in GeometricObject
  // @see app/core/parser/file-parser.js
  // @see app/core/parser/parser-orchestrator.js
  static do (map, vecIdObj, list) {
    const id = map.get("id");
    const pointAId = map.get("param")[0];
    const pointBId = map.get("param")[1];
    // const pointA = list.find(x => x.id === pointAId);
    // const pointB = list.find(x => x.id === pointBId);
    const index_pointA = LineSegmentModel.findOD_LS(vecIdObj, pointAId); // get the index of vecIdObj[ii].id == pointAId
    const index_pointB = LineSegmentModel.findOD_LS(vecIdObj, pointBId); // get the index of vecIdObj[ii].id == pointBId
    const pointA = list[index_pointA]; // pointA = vecIdObj[index_pointA].obj; aux4 += "5,";
    const pointB = list[index_pointB]; // pointB = vecIdObj[index_pointB].obj; aux4 += "6,";

    const label = map.get("label")[0];
    return new LineSegmentModel(pointA, pointB, label, id);
    }

  } // export class LineSegmentModel extends GeometricObject