/*
 * iGeom by LInE
 * Free software to student private data
 *
 * http://www.matematica.br/igeom
 * http://www.usp.br/line
 *
 * Prepare string describing (in GEO format) each objects in iGeomJS
 * 
 * ./app/core/models/application/actions/action.js
 * @version 2023/07/14: Fixed reading GEO file with midpoint
 */

import { GenericObject } from "../../objects/generic-object";
import { GEO_FILE } from "../../../enums/geo-file-enum";

export class Action {

  constructor (genericObject) {
    var auxE = "1, ";
    try {
      this.id = genericObject.id; auxE += "2, "; //D
      this.type = genericObject.elementClass; auxE += "3, "; //D
      this.definition = this.r(genericObject.definitions.map(x => x.id).toString()); auxE += "4, "; //D
      this.list = this.r(genericObject.dependentsOnThis.map(x => x.id).toString()); auxE += "5, "; //D
      this.label = genericObject.labelIgeom; auxE += "6, "; //D
      this.defined = genericObject.defined; auxE += "7, "; //D
      this.color = genericObject.color; auxE += "8, "; //D
      this.hidden = genericObject.visible ? 0 : 1; auxE += "9, "; //D
      this.pixel = "";
      this.font = "";
      this.labelColor = genericObject.labelColor; auxE += "10, "; //D
      this.genericObject = genericObject;
    } catch (error) {
      console.log("action.js!constructor(genericObject): Error '" + error + "', with " + auxE);
      }
    console.log("action.js!constructor(genericObject): this.id=" + (genericObject.id!=undefined ? genericObject.id : "<>") + " final"); //D
    }

  do () {
    //D console.log("action.js!do(): ");
    const genericObject = new GenericObject(this.id);
    genericObject.color = this.color;
    genericObject.elementClass = this.type;
    genericObject.definition = this.definition;
    genericObject.list = this.list;
    genericObject.label = this.label;
    genericObject.visible = this.hidden == 0;
    }

  toMap () {
    const map = new Map();
    map.set("key", "value");
    return map;
    }

  fromMapList (map) { }

  toString () {
    //  Items of GEO line: Id, Type, Definition, List, Label, Defined, Color, Hidden, Pixel, Font, LabelColor;
    //  Correspon. values: 1   0     2           3     4      5        6      7       8      9     10
    var strC = "", strId = "", tam;
    try {
    if (this.id!=undefined) strC += `${this.d(GEO_FILE.ID, this.id)}, `;
    strId = strC;
    strC += `${this.d(GEO_FILE.TYPE, this.type)}, `;                                  // TYPE is 0 and JavaScript interpret it as false (do not use "if")
    if (this.definition) strC += `${this.d(GEO_FILE.DEFINITION, this.definition)}, `;
    if (this.list)       strC += `${this.d(GEO_FILE.LIST, this.list)}, `;
    if (this.label)      strC += `${this.d(GEO_FILE.LABEL, this.label)}, `;
    if (this.defined)    strC += `${this.d(GEO_FILE.DEFINED, this.defined)}, `;
    if (this.color)      strC += `${this.d(GEO_FILE.COLOR, this.color)}, `;
    if (this.hidden)     strC += `${this.d(GEO_FILE.HIDDEN, this.hidden)}, `;
    else                 strC += `${this.d(GEO_FILE.HIDDEN, 0)}, `;                   // Must set "7:0" otherwise the object will be hidden
    if (this.pixel)      strC += `${this.d(GEO_FILE.PIXEL, this.pixel)}, `;
    if (this.font)       strC += `${this.d(GEO_FILE.FONT, this.font)}, `;
    if (this.labelColor) strC += `${this.d(GEO_FILE.LABEL_COLOR, this.labelColor)}`;
    strC = strC.trim();
    tam = strC.length;
    if (strC[tam-1] == ",") 
      strC = strC.substring(0, tam-1).trim();
    } catch (error) { console.log("action.js!toString(): error " + error); }
    strC = "{" + strC + "}!\n";
    console.log("action.js!toString(): id=" + strId + "\n" + strC);
    //D console.trace();
    return strC;
    } // toString()

  d (key, value) { //TODO this name must be improved!
    return `${key}:${value}`;
    }

  r (str) { //TODO this name must be improved!
    return str.replace(/,/g, " ");
    }

  rehydrate () {
    console.log("action.js!rehydrate(): inicial - this.genericObject.id=" + this.genericObject.id); //D
    this.id = this.genericObject.id; // Attention, caution with ID=0
    this.type = this.genericObject.elementClass;
    this.definition = this.r(this.genericObject.definitions.map(x => x.id).toString());
    this.list = this.r(this.genericObject.dependentsOnThis.map(x => x.id).toString());
    this.label = this.genericObject.labelIgeom;
    this.defined = this.genericObject.defined;
    this.color = this.genericObject.color;
    this.hidden = this.genericObject.visible ? 0 : 1;
    this.pixel = "";
    this.font = "";
    this.labelColor = "";
    console.log("action.js!rehydrate(): final"); //D
    }

  } // export class Action