// iGeom JS
// LInE: "free software and private data"
// http://www.matematica.br
// http://www.usp.br/line

import * as ObjectClass from "./../enums/elements-class-enum";

const TYPE = 0; // identifica se Ponto, Texto, Reta...
const ID = 1; // identificador unico do objeto
const DEFINITION = 2; // parametros dos quais o objeto depende (exemplo Reta depende de Ponto e Ponto => "2:IDPONTO1 IDPONTO2")
const LIST = 3; // lista de identificadores de objetos dependentes do atual
const LABEL = 4; // definicao do rotulo (valor/nome e se escondido ou nao)
const DEFINED = 6; // 0 ou 1 (???)
const COLOR = 5; // cor usando valor inteiro
const HIDDEN = 7; // 0 (visivel) ou 1 (escondido)
const PIXEL = 8; // LeScript usa (coordenada X e Y - inteiros)
const FONT = 9; // 'Arial' T Tam: T = FONT.PLAIN, FONT.BOLD, FONT.ITALIC; Tam = 8, 9,...
// PROIBIDO: "Arial" T Tam, pois em caso contrario nao sera lido como parametro
//           usar MARCAFNT = "\'" nao pode ser "\"" <[28/09/2006] agora vazio "">
const LABEL_COLOR = 10; // cor de rotulo (tambem um inteiro) - inserido apos iGeom versao >= 2.9.9.31

const OBJECT_LINE_REGEX = /^\{\w*(?:(?:\d+:@\|[^@]+@\|)|(?:\d+:[^,@|]+))\w(?:,\w*(?:(?:\d+:@\|[^@]+@\|)|(?:\d+:[^,@\|]+)))+\w*\}!?$/.compile();
const ALPHA_MASK = 255 << 24;

export class FileParser {

  // @param {string} content
  constructor (content) {
    this.content = content; // String with GEO file content
    this.lines = []; // to store each GEO line (help debug)
    }

  //DEBUG: get String from map to check associated geometric object
  static printMap_fp (map) {
    var str = ""; // map.forEach((value: boolean, key: string) => { str += "[" + key + "=" + value + "] "; });
    for (let [key, value] of map) { str += "[" + key + "=" + value + "] "; }
    return str;
    }

  // Analyse GEO content building a Map with objects
  // @param vec_OD is Array with entries {id:<value>, obj:<dynamic object>}
  // @calledby app/core/application/header-menu.js: onFileChanged()
  parse (vec_OD) {
    console.log("file-parser.js!parse(): start");
    var parsed_object, obj, parsed_result = this.content.split("\n").filter(str => !str.match(/^[#\[]/))
      .map(str => {
        try {
        var aux_vec, only_id = -1; // integer with ID only (to debug proces) returned by parseObjectLine(object_line): return [map, only_id];
        this.lines.push(str);
        if (!OBJECT_LINE_REGEX.test(str.trim())) {
          console.error("file-parser.js!parse(.): Malformed object string in iGeom file: ", str);
          return undefined;
          }
        only_id = -1; // security to debug
        aux_vec = this.parseObjectLine(str.trim()); // go to parseObjectLine(.)
        if (aux_vec && aux_vec.length>1) { parsed_object = aux_vec[0]; only_id = aux_vec[1]; } else parsed_object = aux_vec;
        if (parsed_object && parsed_object!="" &&  parsed_object!=undefined && only_id!=-1) {
          try { // file-parser.js!parse(): vec_id.push error TypeError: parsed_object.get is not a function::""
            obj = { id : only_id, obj : parsed_object };
            vec_OD.push(obj); // by reference, change "effective parameter"
            //vec_id.push(parsed_object.get("id"));
          } catch (error) { console.log("file-parser.js!parse(): vec_id.push error " + error + "::" + JSON.stringify(parsed_object)); }
          }
        //D console.log("file-parser.js!parse(): str=" + str + "\nparsed_object=" + JSON.stringify(parsed_object)); //D
        return parsed_object;
        } catch (error) {
          // console.error("file-parser.js!parse(): error " + error + " in: " + str);
          console.log("file-parser.js!parse(): error " + error + " in: " + str);
          }
        });
    // var aux1 = ""; try { aux1 = JSON.stringify(Object.keys(parsed_result)) + " "; } catch (error) { aux1 = "error " + error + " "; }
    var aux1 = ""; try { aux1 = parsed_result + " "; } catch (error) { aux1 = "error " + error + " "; }
    var aux2 = ""; try { var tam = vec_OD.length; for (var ii=0; ii<tam; ii++) aux2 += "[" + vec_OD[ii].id + ":" + vec_OD[ii].obj + "],"; } catch (error) { aux2 += "error " + error + " "; }
    console.log("file-parser.js!parse(): parsed_result=" + aux1); //D
    console.log("file-parser.js!parse(): vec_OD[]=" + aux2 + "\n* final"); //D
    return parsed_result;
    } // parse(vec_OD)

  // Function to analyse each line of a GEO file, currently with 8 itens.
  // A typical GEO line (to free Point) and its 8 itens:
  //   1:0, 0:0, 2:215.0 -145.0, 3:2, 4:A 4.0 -10.0 0, 6:1, 5:-16711936, 7:0
  // in this line, following the order in line, the param code are defined in "app/core/enums/geo-file-enum.js":
  //  ID=1, TYPE=0, DEFINITION=2, LIST=3, LABEL=4, COLOR=5, DEFINED=6, HIDDEN=7, PIXEL=8, FONT=9, LABEL_COLOR=10
  //  E.g: "1" ("1:0"),  "0" ("0:0"),  "2" ("2:215.0 -145.0"), "3" ("3:2"), "4" ("4:A 4.0 -10.0 0"), 
  //       "6" ("6:1"), "5" ("5:-16711936") and "7" ("7:0")
  // @param {string} object_line
  parseObjectLine (object_line) {
    var only_id = -1; // integer with ID only (to debug proces)
    const map = new Map();
    object_line = object_line.substring(1); // remove initial character '{'
    if (object_line.endsWith("}!")) {
      object_line = object_line.substring(0, object_line.length - 2); // remove final char '}!'
    } else {
      object_line = object_line.substring(0, object_line.length - 1); // remove final char '}'
      }
    if (!object_line || object_line.length==0) return "";
    console.log("file-parser.js: object_line = '" + object_line + "'"); //D
    var str2Debug = ""; //D13
    var prop, prop_info, key, type, id, value_id, definition_param_list, list, param, maybe_number,
        id_list, array_list, label_param_list, defined_value, color_value,
        hidden_value, pixel_position, font_param_list, label_color_value, label_color;
    const properties = object_line.split(",");
    for (const index in properties) {
      prop = properties[index];
      prop_info = prop.trim().split(":");
      key = parseInt(prop_info[0]);
      if (prop_info[1] == undefined) continue;
      if (key == TYPE) { // TYPE = 0
        // TYPE = {0 => Point; 1=>IntersectionPoint; 2=>PointOver; 3=>Circumference; 4=>StraightLine; 5=>Ray; 6=>Segment; ... },
        type = prop_info[1];
        str2Debug += "TYPE=" + key + ": " + type + ", "; //D13
        map.set("type", ObjectClass.fromCodeToClass(parseInt(type)));
      } else if (key == ID) { // ID = 0
        id = prop_info[1];
        value_id = parseInt(id);
        str2Debug += "ID=" + key + ": " + (value_id+0) + ", "; //D13 "value_id+0" to identify error in "integer"
        //D console.log("file-parser.js: ID: id=" + id + " = " + prop_info[1]); // is ID
	only_id = value_id; // integer with ID only (to debug proces)
        map.set("id", value_id);
      } else if (key == DEFINITION) { // DEFINITION = 2: "2:215.0 -145.0" or "2:0 1"...
        var aux = ""; //D_
        str2Debug += "DEFINITION=" + key + ": ("; //D13
        definition_param_list = [];
        const list = prop_info[1].trim().split(" ");
        aux += "index=" + index + ", #prop_info[1]=" + list.length + ", "; //D_
        var ii=0; //D_
        for (const i in list) {
          const param = list[i];
          const maybe_number = Number(param);
        str2Debug += param + ", "; //D13
          aux += "(" + ii + "," + param + " == " + maybe_number + "?); "; //D_
          ii++; //D_
          if (Number.isNaN(maybe_number)) {
            definition_param_list.push(param);
          } else {
            definition_param_list.push(maybe_number);
            }
          } // for (const i in list)
        //DD console.log("file-parser.js: DEFINITION: " + aux); //D_
        map.set("param", definition_param_list);
        str2Debug += "), "; //D13
      } else if (key == LIST) { // LIST = 3
        str2Debug += "LIST=" + key + ": ("; //D13
        id_list = [];
        array_list = prop_info[1].trim().split(" ");
        for (const i in array_list) {
          const id1 = array_list[i];
          str2Debug += id1 + ", "; //D13
          id_list.push(id1);
          }
        str2Debug += "), "; //D13
        map.set("deps", id_list);
      } else if (key == LABEL) { // LABEL = 4
        str2Debug += "LABEL=" + key + ": ("; //D13
        label_param_list = [];
        const list = prop_info[1].trim().split(" ");
        for (const i in list) {
          const param = list[i];
        str2Debug += param + ", "; //D13
          const maybe_number = Number(param);
          if (Number.isNaN(maybe_number)) {
            label_param_list.push(param);
          } else {
            label_param_list.push(maybe_number);
            }
          }
        str2Debug += "), "; //D13
        map.set("label", label_param_list);
      } else if (key == DEFINED) { // DEFINED = 6
        defined_value = prop_info[1];
        str2Debug += "DEFINED=" + key + ": " + defined_value + ", "; //D13
        map.set("defined", parseInt(defined_value));
      } else if (key == COLOR) { // COLOR = 5
        color_value = prop_info[1];
        str2Debug += "COLOR=" + key + ": " + color_value + ", "; //D13
        let number = Number(color_value);
        const alpha = ((number & ALPHA_MASK) >>> 24) / 255;
        number = number ^ ALPHA_MASK;
        const r = (number >> 16) & 0xff;
        const g = (number >> 8) & 0xff;
        const b = number & 0xff;
        map.set("color", { alpha: alpha, r: r, g: g, b: b });
      } else if (key == HIDDEN) { // HIDDEN = 7
        hidden_value = prop_info[1];
        str2Debug += "HIDDEN=" + key + ": " + hidden_value + ", "; //D13
        if (hidden_value) // optional: if empty do not insert this property
          map.set("hidden", parseInt(hidden_value));
      } else if (key == PIXEL) { // PIXEL = 8
        str2Debug += "PIXEL=" + key + ": ("; //D13
        pixel_position = [];
        const list = prop_info[1].trim().split(" ");
        var count_pix = 0;
        for (const index in list) {
          const value_pos = list[index];
          const value_pos_int = parseInt(value_pos);
          str2Debug += value_pos + ", "; //D13
          if (value_pos_int) { // optional: if empty do not insert this property
            count_pix++;
            pixel_position.push(value_pos_int);
	    }
          }
        str2Debug += "), "; //D13
        if (count_pix>0) // optional: if empty do not insert this property
          map.set("position", pixel_position);
        console.log("file-parser.js!parseObjectLine(): PIXEL=" + PIXEL + ", count_pix=" + count_pix);
      } else if (key == FONT) { // FONT = 9
        font_param_list = prop_info[1].trim().split(" ");
        if (font_param_list && font_param_list.length>0) // optional: if empty do not insert this property
          map.set("font", font_param_list);
        str2Debug += "FONT=" + key + ": " + font_param_list + ", "; //D13
      } else if (key == LABEL_COLOR) { // LABEL_COLOR = 10
        label_color_value = prop_info[1];
        str2Debug += "LABEL_COLOR=" + key + ": (" + label_color_value + "="; //D13
        let number = Number(label_color_value);
        const alpha = ((number & ALPHA_MASK) >>> 24) / 255;
        number = number ^ ALPHA_MASK;
        const r = (number >> 16) & 0xff;
        const g = (number >> 8) & 0xff;
        const b = number & 0xff;
        str2Debug += "RGB("+r+","+g+","+b+")), "; //D13
        if (r && g && b) // optional: if empty do not insert this property
          map.set("label_color", { alpha: alpha, r: r, g: g, b: b });
        label_color = "(" + alpha + ", RGB[" + r + "," + g + "," + b + "])";
        }
      } // for (const index in properties)
    console.log("file-parser.js: '" + str2Debug + "'"); //D
    if (type==13) { //D
      var auxDef = "";
      for (var ii=0; ii<definition_param_list.length; ii++) auxDef += definition_param_list[ii] + ",";
      console.log("file-parser.js!parseObjectLine(.): id=" + id + ", DEFINITION=(" + auxDef + ")");
      console.log("file-parser.js!parseObjectLine(.): id=" + id + ", map=(" + FileParser.printMap_fp(map) + ")"); // JSON.stringify(Object.keys ...) // map.entries()
      // map.entries() = [object Map Iterator]
      }
    var vec_aux;
    try { // build Array() with {id:<value>, obj:<dynamic object>}
      vec_aux = [map, only_id];
    } catch (error) { vec_aux = map; console.log("file-parser.js!parseObjectLine(.): id=" + id + ", error " + error); }
    return vec_aux;
    } // parseObjectLine(object_line)

  } // export class FileParser